LKML Archive on lore.kernel.org
help / color / mirror / Atom feed
From: Josh Poimboeuf <jpoimboe@redhat.com>
To: Peter Zijlstra <peterz@infradead.org>
Cc: Kairui Song <kasong@redhat.com>, Alexei Starovoitov <ast@fb.com>,
Song Liu <songliubraving@fb.com>,
lkml <linux-kernel@vger.kernel.org>,
Kernel Team <Kernel-team@fb.com>,
Alexei Starovoitov <ast@kernel.org>,
Daniel Borkmann <daniel@iogearbox.net>,
"bpf@vger.kernel.org" <bpf@vger.kernel.org>,
Steven Rostedt <rostedt@goodmis.org>
Subject: Re: Getting empty callchain from perf_callchain_kernel()
Date: Thu, 13 Jun 2019 15:26:13 -0500 [thread overview]
Message-ID: <20190613202613.zt4rvxiqyaolvqpq@treble> (raw)
In-Reply-To: <20190612145008.3l5iguuwk2termi4@treble>
On Wed, Jun 12, 2019 at 09:50:08AM -0500, Josh Poimboeuf wrote:
> > Other than that, the same note as before, the 32bit JIT still seems
> > buggered, but I'm not sure you (or anybody else) cares enough about that
> > to fix it though. It seems to use ebp as its own frame pointer, which
> > completely defeats an unwinder.
>
> I'm still trying to decide if I care about 32-bit. It does indeed use
> ebp everywhere. But I'm not sure if I want to poke the beehive... Also
> factoring into the equation is the fact that I'll be on PTO next week
> :-) If I have time in the next couple days then I may take a look.
32-bit actually looks much easier to fix than 64-bit was. I haven't
tested it yet though, but I'll be gone next week so I'll just drop it
here in case anybody wants to try it.
diff --git a/arch/x86/net/bpf_jit_comp32.c b/arch/x86/net/bpf_jit_comp32.c
index b29e82f190c7..8c1de7786e49 100644
--- a/arch/x86/net/bpf_jit_comp32.c
+++ b/arch/x86/net/bpf_jit_comp32.c
@@ -169,6 +169,10 @@ static const u8 bpf2ia32[][2] = {
#define src_hi src[1]
#define STACK_ALIGNMENT 8
+
+/* Size of callee-saved register space (except EBP) */
+#define CALLEE_SAVE_SIZE 12
+
/*
* Stack space for BPF_REG_1, BPF_REG_2, BPF_REG_3, BPF_REG_4,
* BPF_REG_5, BPF_REG_6, BPF_REG_7, BPF_REG_8, BPF_REG_9,
@@ -176,13 +180,14 @@ static const u8 bpf2ia32[][2] = {
*/
#define SCRATCH_SIZE 96
-/* Total stack size used in JITed code */
+/* Total stack size used in JITed code (except callee-saved) */
#define _STACK_SIZE (stack_depth + SCRATCH_SIZE)
#define STACK_SIZE ALIGN(_STACK_SIZE, STACK_ALIGNMENT)
-/* Get the offset of eBPF REGISTERs stored on scratch space. */
-#define STACK_VAR(off) (off)
+/* Offset of eBPF REGISTERs stored in scratch space, relative to EBP */
+//FIXME: rename to EBP_OFFSET
+#define STACK_VAR(off) (off - CALLEE_SAVE_SIZE - SCRATCH_SIZE)
/* Encode 'dst_reg' register into IA32 opcode 'byte' */
static u8 add_1reg(u8 byte, u32 dst_reg)
@@ -1408,7 +1413,7 @@ struct jit_context {
#define BPF_MAX_INSN_SIZE 128
#define BPF_INSN_SAFETY 64
-#define PROLOGUE_SIZE 35
+#define PROLOGUE_SIZE 32
/*
* Emit prologue code for BPF program and check it's size.
@@ -1436,8 +1441,6 @@ static void emit_prologue(u8 **pprog, u32 stack_depth)
/* sub esp,STACK_SIZE */
EMIT2_off32(0x81, 0xEC, STACK_SIZE);
- /* sub ebp,SCRATCH_SIZE+12*/
- EMIT3(0x83, add_1reg(0xE8, IA32_EBP), SCRATCH_SIZE + 12);
/* xor ebx,ebx */
EMIT2(0x31, add_2reg(0xC0, IA32_EBX, IA32_EBX));
@@ -1470,18 +1473,21 @@ static void emit_epilogue(u8 **pprog, u32 stack_depth)
/* mov edx,dword ptr [ebp+off]*/
EMIT3(0x8B, add_2reg(0x40, IA32_EBP, IA32_EDX), STACK_VAR(r0[1]));
- /* add ebp,SCRATCH_SIZE+12*/
- EMIT3(0x83, add_1reg(0xC0, IA32_EBP), SCRATCH_SIZE + 12);
+ /* add esp, STACK_SIZE */
+ EMIT2_off32(0x81, 0xC4, STACK_SIZE);
+
+ /* pop ebx */
+ EMIT1(0x5b);
+ /* pop esi */
+ EMIT1(0x5e);
+ /* pop edi */
+ EMIT1(0x5f);
+ /* pop ebp */
+ EMIT1(0x5d);
- /* mov ebx,dword ptr [ebp-12]*/
- EMIT3(0x8B, add_2reg(0x40, IA32_EBP, IA32_EBX), -12);
- /* mov esi,dword ptr [ebp-8]*/
- EMIT3(0x8B, add_2reg(0x40, IA32_EBP, IA32_ESI), -8);
- /* mov edi,dword ptr [ebp-4]*/
- EMIT3(0x8B, add_2reg(0x40, IA32_EBP, IA32_EDI), -4);
+ /* ret */
+ EMIT1(0xC3);
- EMIT1(0xC9); /* leave */
- EMIT1(0xC3); /* ret */
*pprog = prog;
}
next prev parent reply other threads:[~2019-06-13 20:26 UTC|newest]
Thread overview: 45+ messages / expand[flat|nested] mbox.gz Atom feed top
2019-05-16 23:51 Song Liu
2019-05-17 7:46 ` Peter Zijlstra
2019-05-17 8:10 ` Peter Zijlstra
2019-05-17 8:15 ` Kairui Song
2019-05-17 8:32 ` Kairui Song
2019-05-17 16:22 ` Song Liu
2019-05-17 9:10 ` Peter Zijlstra
2019-05-17 18:40 ` Song Liu
2019-05-17 21:06 ` Alexei Starovoitov
2019-05-17 21:48 ` Song Liu
2019-05-19 18:07 ` Kairui Song
2019-05-20 17:22 ` Song Liu
2019-05-22 13:51 ` Peter Zijlstra
2019-05-19 18:06 ` Kairui Song
2019-05-20 17:16 ` Song Liu
2019-05-20 17:19 ` Song Liu
2019-05-22 14:02 ` Peter Zijlstra
2019-05-22 14:49 ` Alexei Starovoitov
2019-05-22 17:45 ` Josh Poimboeuf
2019-05-22 23:46 ` Josh Poimboeuf
2019-05-23 6:48 ` Kairui Song
2019-05-23 8:27 ` Song Liu
2019-05-23 9:11 ` Kairui Song
2019-05-23 13:32 ` Josh Poimboeuf
2019-05-23 14:50 ` Kairui Song
2019-05-23 15:24 ` Josh Poimboeuf
2019-05-23 16:41 ` Kairui Song
2019-05-23 17:27 ` Josh Poimboeuf
2019-05-24 2:20 ` Kairui Song
2019-05-24 23:23 ` Josh Poimboeuf
2019-05-27 11:57 ` Kairui Song
2019-06-06 16:04 ` Song Liu
2019-06-06 23:58 ` Josh Poimboeuf
2019-06-11 21:03 ` Josh Poimboeuf
2019-05-24 8:53 ` Peter Zijlstra
2019-05-24 13:05 ` Josh Poimboeuf
2019-06-12 3:05 ` Josh Poimboeuf
2019-06-12 8:54 ` Peter Zijlstra
2019-06-12 14:50 ` Josh Poimboeuf
2019-06-13 20:26 ` Josh Poimboeuf [this message]
2019-06-12 13:10 ` Steven Rostedt
2019-06-12 14:26 ` Josh Poimboeuf
2019-05-22 18:07 ` Josh Poimboeuf
2019-05-22 21:55 ` Alexei Starovoitov
2019-05-17 16:32 ` Song Liu
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20190613202613.zt4rvxiqyaolvqpq@treble \
--to=jpoimboe@redhat.com \
--cc=Kernel-team@fb.com \
--cc=ast@fb.com \
--cc=ast@kernel.org \
--cc=bpf@vger.kernel.org \
--cc=daniel@iogearbox.net \
--cc=kasong@redhat.com \
--cc=linux-kernel@vger.kernel.org \
--cc=peterz@infradead.org \
--cc=rostedt@goodmis.org \
--cc=songliubraving@fb.com \
--subject='Re: Getting empty callchain from perf_callchain_kernel()' \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).